@@ -46,7 +46,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.8.1"
+__version__ = "0.9"
 
 import math
 
@@ -183,7 +183,7 @@ class Collider(sge.Object):
             if stopper is not None:
                 move_loss = max(0, abs(move) - abs(self.x - old_x))
                 self.event_physics_collision_right(stopper, move_loss)
-                stopper.event_physics_collision_left(self)
+                stopper.event_physics_collision_left(self, 0)
                 
         elif move < 0:
             bbb = round(self.bbox_bottom, NDIG)
@@ -269,7 +269,7 @@ class Collider(sge.Object):
             if stopper is not None:
                 move_loss = max(0, abs(move) - abs(self.x - old_x))
                 self.event_physics_collision_left(stopper, move_loss)
-                stopper.event_physics_collision_right(self)
+                stopper.event_physics_collision_right(self, 0)
 
         # Engage stickiness (same whether moving left or right)
         # 1 = sticking to the floor
@@ -278,7 +278,13 @@ class Collider(sge.Object):
             if (not self.get_bottom_touching_slope() and
                     not self.get_bottom_touching_wall()):
                 new_bbox_bottom = None
-                for other in sge.game.current_room.objects:
+                others = (
+                    sge.game.current_room.get_objects_at(
+                        self.bbox_left, self.bbox_top, self.bbox_width,
+                        (sge.game.current_room.height - self.bbox_top +
+                         sge.game.current_room.object_area_height)) |
+                    sge.game.current_room.object_area_void)
+                for other in others:
                     if (other.bbox_left >= self.bbox_right or
                             other.bbox_right <= self.bbox_left):
                         continue
@@ -303,7 +309,11 @@ class Collider(sge.Object):
             if (not self.get_top_touching_slope() and
                     not self.get_top_touching_wall()):
                 new_bbox_top = None
-                for other in sge.game.current_room.objects:
+                others = (
+                    sge.game.current_room.get_objects_at(
+                        self.bbox_left, 0, self.bbox_width, self.bbox_bottom) |
+                    sge.game.current_room.object_area_void)
+                for other in others:
                     if (other.bbox_left >= self.bbox_right or
                             other.bbox_right <= self.bbox_left):
                         continue
@@ -434,7 +444,7 @@ class Collider(sge.Object):
             if stopper is not None:
                 move_loss = max(0, abs(move) - abs(self.y - old_y))
                 self.event_physics_collision_bottom(stopper, move_loss)
-                stopper.event_physics_collision_top(self)
+                stopper.event_physics_collision_top(self, 0)
                 
         elif move < 0:
             bbr = round(self.bbox_right, NDIG)
@@ -520,7 +530,7 @@ class Collider(sge.Object):
             if stopper is not None:
                 move_loss = max(0, abs(move) - abs(self.y - old_y))
                 self.event_physics_collision_top(stopper, move_loss)
-                stopper.event_physics_collision_bottom(self)
+                stopper.event_physics_collision_bottom(self, 0)
 
         # Engage stickiness (same whether moving left or right)
         # 1 = sticking to a wall on the right
@@ -529,7 +539,14 @@ class Collider(sge.Object):
             if (not self.get_right_touching_slope() and
                     not self.get_right_touching_wall()):
                 new_bbox_right = None
-                for other in sge.game.current_room.objects:
+                others = (
+                    sge.game.current_room.get_objects_at(
+                        self.bbox_left, self.bbox_top,
+                        (sge.game.current_room.width - self.bbox_left +
+                         sge.game.current_room.object_area_width),
+                        self.bbox_height) |
+                    sge.game.current_room.object_area_void)
+                for other in others:
                     if (other.bbox_top >= self.bbox_bottom or
                             other.bbox_bottom <= self.bbox_top):
                         continue
@@ -553,7 +570,11 @@ class Collider(sge.Object):
             if (not self.get_left_touching_slope() and
                     not self.get_left_touching_wall()):
                 new_bbox_left = None
-                for other in sge.game.current_room.objects:
+                others = (
+                    sge.game.current_room.get_objects_at(
+                        0, self.bbox_top, self.bbox_right, self.bbox_height) |
+                    sge.game.current_room.object_area_void)
+                for other in others:
                     if (other.bbox_top >= self.bbox_bottom or
                             other.bbox_bottom <= self.bbox_top):
                         continue
@@ -771,7 +792,7 @@ class SolidLeft(sge.Object):
     from the top.
     """
 
-    def event_physics_collision_left(self, other):
+    def event_physics_collision_left(self, other, move_loss):
         """
         Called when the left side of the wall collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -788,7 +809,7 @@ class SolidRight(sge.Object):
     from the right.
     """
 
-    def event_physics_collision_right(self, other):
+    def event_physics_collision_right(self, other, move_loss):
         """
         Called when the right side of the wall collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -805,7 +826,7 @@ class SolidTop(sge.Object):
     from the top.
     """
 
-    def event_physics_collision_top(self, other):
+    def event_physics_collision_top(self, other, move_loss):
         """
         Called when the top side of the wall collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -822,7 +843,7 @@ class SolidBottom(sge.Object):
     from the bottom.
     """
 
-    def event_physics_collision_bottom(self, other):
+    def event_physics_collision_bottom(self, other, move_loss):
         """
         Called when the bottom side of the wall collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -868,7 +889,7 @@ class SlopeTopLeft(sge.Object):
 
     def get_slope_x(self, y):
         """
-        Get the corresponding y coordinate of a given x coordinate for
+        Get the corresponding x coordinate of a given y coordinate for
         the slope.
         """
         # x = (y - b) / m [b is 0]
@@ -878,7 +899,7 @@ class SlopeTopLeft(sge.Object):
 
     def get_slope_y(self, x):
         """
-        Get the corresponding x coordinate of a given y coordinate for
+        Get the corresponding y coordinate of a given x coordinate for
         the slope.
         """
         # y = mx + b [b is 0]
@@ -886,7 +907,7 @@ class SlopeTopLeft(sge.Object):
         x -= self.bbox_left
         return m * x + self.bbox_bottom
 
-    def event_physics_collision_left(self, other):
+    def event_physics_collision_left(self, other, move_loss):
         """
         Called when the left side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -895,7 +916,7 @@ class SlopeTopLeft(sge.Object):
         """
         pass
 
-    def event_physics_collision_top(self, other):
+    def event_physics_collision_top(self, other, move_loss):
         """
         Called when the top side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -932,7 +953,7 @@ class SlopeTopRight(sge.Object):
 
     def get_slope_x(self, y):
         """
-        Get the corresponding y coordinate of a given x coordinate for
+        Get the corresponding x coordinate of a given y coordinate for
         the slope.
         """
         # x = (y - b) / m [b is 0]
@@ -942,7 +963,7 @@ class SlopeTopRight(sge.Object):
 
     def get_slope_y(self, x):
         """
-        Get the corresponding x coordinate of a given y coordinate for
+        Get the corresponding y coordinate of a given x coordinate for
         the slope.
         """
         # y = mx + b [b is 0]
@@ -950,7 +971,7 @@ class SlopeTopRight(sge.Object):
         x -= self.bbox_left
         return m * x + self.bbox_top
 
-    def event_physics_collision_right(self, other):
+    def event_physics_collision_right(self, other, move_loss):
         """
         Called when the right side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -959,7 +980,7 @@ class SlopeTopRight(sge.Object):
         """
         pass
 
-    def event_physics_collision_top(self, other):
+    def event_physics_collision_top(self, other, move_loss):
         """
         Called when the top side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -996,7 +1017,7 @@ class SlopeBottomLeft(sge.Object):
 
     def get_slope_x(self, y):
         """
-        Get the corresponding y coordinate of a given x coordinate for
+        Get the corresponding x coordinate of a given y coordinate for
         the slope.
         """
         # x = (y - b) / m [b is 0]
@@ -1006,7 +1027,7 @@ class SlopeBottomLeft(sge.Object):
 
     def get_slope_y(self, x):
         """
-        Get the corresponding x coordinate of a given y coordinate for
+        Get the corresponding y coordinate of a given x coordinate for
         the slope.
         """
         # y = mx + b [b is 0]
@@ -1014,7 +1035,7 @@ class SlopeBottomLeft(sge.Object):
         x -= self.bbox_left
         return m * x + self.bbox_top
 
-    def event_physics_collision_left(self, other):
+    def event_physics_collision_left(self, other, move_loss):
         """
         Called when the left side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -1023,7 +1044,7 @@ class SlopeBottomLeft(sge.Object):
         """
         pass
 
-    def event_physics_collision_bottom(self, other):
+    def event_physics_collision_bottom(self, other, move_loss):
         """
         Called when the bottom side of the slope collides with a
         collider in the sense of the physics system, rather than in the
@@ -1078,7 +1099,7 @@ class SlopeBottomRight(sge.Object):
         x -= self.bbox_left
         return m * x + self.bbox_bottom
 
-    def event_physics_collision_right(self, other):
+    def event_physics_collision_right(self, other, move_loss):
         """
         Called when the right side of the slope collides with a collider
         in the sense of the physics system, rather than in the sense of
@@ -1087,7 +1108,7 @@ class SlopeBottomRight(sge.Object):
         """
         pass
 
-    def event_physics_collision_bottom(self, other):
+    def event_physics_collision_bottom(self, other, move_loss):
         """
         Called when the bottom side of the slope collides with a
         collider in the sense of the physics system, rather than in the
@@ -1243,15 +1264,15 @@ class MobileWall(sge.Object):
         old_x = self.x
         self.x += move
         for other in stuck:
-            other.move_y(move, True)
+            other.move_x(move, True)
 
         if move > 0:
             if isinstance(self, SolidRight):
                 for other in self.collision(Collider):
                     if not self.collision(other, x=old_x):
                         other.move_x(self.bbox_right - other.bbox_left, True)
-                        self.event_physics_collision_right(other)
-                        other.event_physics_collision_left(self)
+                        self.event_physics_collision_right(other, 0)
+                        other.event_physics_collision_left(self, 0)
             if isinstance(self, SlopeTopRight):
                 for other in self.collision(Collider):
                     x = self.get_slope_x(other.bbox_bottom)
@@ -1260,13 +1281,13 @@ class MobileWall(sge.Object):
                             other.move_x(x - other.bbox_left, True)
                             y = self.get_slope_y(other.bbox_left)
                             other.move_y(y - other.bbox_bottom, True)
-                            self.event_physics_collision_right(other)
-                            other.event_physics_collision_left(self)
+                            self.event_physics_collision_right(other, 0)
+                            other.event_physics_collision_left(self, 0)
                         elif not self.collision(other, x=old_x):
                             other.move_x(self.bbox_right - other.bbox_left,
                                          True)
-                            self.event_physics_collision_right(other)
-                            other.event_physics_collision_left(self)
+                            self.event_physics_collision_right(other, 0)
+                            other.event_physics_collision_left(self, 0)
             if isinstance(self, SlopeBottomRight):
                 for other in self.collision(Collider):
                     x = self.get_slope_x(other.bbox_top)
@@ -1275,21 +1296,21 @@ class MobileWall(sge.Object):
                             other.move_x(x - other.bbox_left, True)
                             y = self.get_slope_y(other.bbox_left)
                             other.move_y(y - other.bbox_top, True)
-                            self.event_physics_collision_right(other)
-                            other.event_physics_collision_left(self)
+                            self.event_physics_collision_right(other, 0)
+                            other.event_physics_collision_left(self, 0)
                         elif not self.collision(other, x=old_x):
                             other.move_x(self.bbox_right - other.bbox_left,
                                          True)
-                            self.event_physics_collision_right(other)
-                            other.event_physics_collision_left(self)
+                            self.event_physics_collision_right(other, 0)
+                            other.event_physics_collision_left(self, 0)
 
         elif move < 0:
             if isinstance(self, SolidLeft):
                 for other in self.collision(Collider):
                     if not self.collision(other, x=old_x):
                         other.move_x(self.bbox_left - other.bbox_right, True)
-                        self.event_physics_collision_left(other)
-                        other.event_physics_collision_right(self)
+                        self.event_physics_collision_left(other, 0)
+                        other.event_physics_collision_right(self, 0)
             if isinstance(self, SlopeTopLeft):
                 for other in self.collision(Collider):
                     x = self.get_slope_x(other.bbox_bottom)
@@ -1298,13 +1319,13 @@ class MobileWall(sge.Object):
                             other.move_x(x - other.bbox_right, True)
                             y = self.get_slope_y(other.bbox_right)
                             other.move_y(y - other.bbox_bottom, True)
-                            self.event_physics_collision_left(other)
-                            other.event_physics_collision_right(self)
+                            self.event_physics_collision_left(other, 0)
+                            other.event_physics_collision_right(self, 0)
                         elif not self.collision(other, x=old_x):
                             other.move_x(self.bbox_left - other.bbox_right,
                                          True)
-                            self.event_physics_collision_left(other)
-                            other.event_physics_collision_right(self)
+                            self.event_physics_collision_left(other, 0)
+                            other.event_physics_collision_right(self, 0)
             if isinstance(self, SlopeBottomLeft):
                 for other in self.collision(Collider):
                     x = self.get_slope_x(other.bbox_top)
@@ -1313,13 +1334,13 @@ class MobileWall(sge.Object):
                             other.move_x(x - other.bbox_right, True)
                             y = self.get_slope_y(other.bbox_right)
                             other.move_y(y - other.bbox_top, True)
-                            self.event_physics_collision_left(other)
-                            other.event_physics_collision_right(self)
+                            self.event_physics_collision_left(other, 0)
+                            other.event_physics_collision_right(self, 0)
                         elif not self.collision(other, x=old_x):
                             other.move_x(self.bbox_left - other.bbox_right,
                                          True)
-                            self.event_physics_collision_left(other)
-                            other.event_physics_collision_right(self)
+                            self.event_physics_collision_left(other, 0)
+                            other.event_physics_collision_right(self, 0)
 
     def move_y(self, move):
         """
@@ -1340,8 +1361,8 @@ class MobileWall(sge.Object):
                 for other in self.collision(Collider):
                     if not self.collision(other, y=old_y):
                         other.move_y(self.bbox_bottom - other.bbox_top, True)
-                        self.event_physics_collision_bottom(other)
-                        other.event_physics_collision_top(self)
+                        self.event_physics_collision_bottom(other, 0)
+                        other.event_physics_collision_top(self, 0)
             if isinstance(self, SlopeBottomLeft):
                 for other in self.collision(Collider):
                     y = self.get_slope_y(other.bbox_right)
@@ -1350,13 +1371,13 @@ class MobileWall(sge.Object):
                             other.move_y(y - other.bbox_top, True)
                             x = self.get_slope_x(other.bbox_top)
                             other.move_x(x - other.bbox_right, True)
-                            self.event_physics_collision_bottom(other)
-                            other.event_physics_collision_top(self)
+                            self.event_physics_collision_bottom(other, 0)
+                            other.event_physics_collision_top(self, 0)
                         elif not self.collision(other, y=old_y):
                             other.move_y(self.bbox_bottom - other.bbox_top,
                                          True)
-                            self.event_physics_collision_bottom(other)
-                            other.event_physics_collision_top(self)
+                            self.event_physics_collision_bottom(other, 0)
+                            other.event_physics_collision_top(self, 0)
             if isinstance(self, SlopeBottomRight):
                 for other in self.collision(Collider):
                     y = self.get_slope_y(other.bbox_left)
@@ -1365,21 +1386,21 @@ class MobileWall(sge.Object):
                             other.move_y(y - other.bbox_top, True)
                             x = self.get_slope_x(other.bbox_top)
                             other.move_x(x - other.bbox_left, True)
-                            self.event_physics_collision_bottom(other)
-                            other.event_physics_collision_top(self)
+                            self.event_physics_collision_bottom(other, 0)
+                            other.event_physics_collision_top(self, 0)
                         elif not self.collision(other, y=old_y):
                             other.move_y(self.bbox_bottom - other.bbox_top,
                                          True)
-                            self.event_physics_collision_bottom(other)
-                            other.event_physics_collision_top(self)
+                            self.event_physics_collision_bottom(other, 0)
+                            other.event_physics_collision_top(self, 0)
 
         elif move < 0:
             if isinstance(self, SolidTop):
                 for other in self.collision(Collider):
                     if not self.collision(other, y=old_y):
                         other.move_y(self.bbox_top - other.bbox_bottom, True)
-                        self.event_physics_collision_top(other)
-                        other.event_physics_collision_bottom(self)
+                        self.event_physics_collision_top(other, 0)
+                        other.event_physics_collision_bottom(self, 0)
             if isinstance(self, SlopeTopLeft):
                 for other in self.collision(Collider):
                     y = self.get_slope_y(other.bbox_right)
@@ -1388,13 +1409,13 @@ class MobileWall(sge.Object):
                             other.move_y(y - other.bbox_bottom, True)
                             x = self.get_slope_x(other.bbox_bottom)
                             other.move_x(x - other.bbox_right, True)
-                            self.event_physics_collision_top(other)
-                            other.event_physics_collision_bottom(self)
+                            self.event_physics_collision_top(other, 0)
+                            other.event_physics_collision_bottom(self, 0)
                         elif not self.collision(other, y=old_y):
                             other.move_y(self.bbox_top - other.bbox_bottom,
                                          True)
-                            self.event_physics_collision_top(other)
-                            other.event_physics_collision_bottom(self)
+                            self.event_physics_collision_top(other, 0)
+                            other.event_physics_collision_bottom(self, 0)
             if isinstance(self, SlopeTopRight):
                 for other in self.collision(Collider):
                     y = self.get_slope_y(other.bbox_left)
@@ -1403,10 +1424,128 @@ class MobileWall(sge.Object):
                             other.move_y(y - other.bbox_bottom, True)
                             x = self.get_slope_x(other.bbox_bottom)
                             other.move_x(x - other.bbox_left, True)
-                            self.event_physics_collision_top(other)
-                            other.event_physics_collision_bottom(self)
+                            self.event_physics_collision_top(other, 0)
+                            other.event_physics_collision_bottom(self, 0)
                         elif not self.collision(other, y=old_y):
                             other.move_y(self.bbox_top - other.bbox_bottom,
                                          True)
-                            self.event_physics_collision_top(other)
-                            other.event_physics_collision_bottom(self)
+                            self.event_physics_collision_top(other, 0)
+                            other.event_physics_collision_bottom(self, 0)
+
+
+class MobileColliderWall(MobileWall, Collider):
+
+    """
+    A parent class for mobile walls that are also colliders.  See the
+    documentation for :class:`xsge_physics.Collider` and
+    :class:`xsge_physics.MobileWall` for more information.
+
+    .. note::
+
+       Due to the way movement is implemented in this class, it is not
+       safe to move this object during its collision events.  Doing so
+       may cause colliders that should be stuck to this object to become
+       un-stuck, or cause this object to pass through colliders it
+       should be pushing.
+    """
+
+    def move_x(self, move, absolute=False):
+        """
+        Move the wall horizontally, handling physics.
+
+        Arguments:
+
+        - ``move`` -- The amount to add to :attr:`y`.
+        """
+        xprev = self.x
+        Collider.move_x(self, move, absolute)
+        real_move = self.x - xprev
+        self.x = xprev
+        MobileWall.move_x(self, real_move)
+
+    def move_y(self, move, absolute=False):
+        """
+        Move the wall vertically, handling physics.
+
+        Arguments:
+
+        - ``move`` -- The amount to add to :attr:`y`.
+        """
+        yprev = self.y
+        Collider.move_y(self, move, absolute)
+        real_move = self.y - yprev
+        self.y = yprev
+        MobileWall.move_y(self, real_move)
+
+    def event_physics_collision_left(self, other, move_loss):
+        """
+        Called when the left side of the wall collides with a collider,
+        wall, or slope in the sense of the physics system, rather than
+        in the sense of SGE collision detection.  See the documentation
+        for :meth:`xsge_physics.Collider.event_physics_collision_left`
+        for more information.
+
+        .. note::
+
+           Due to the way movement is implemented in this class, it is
+           not safe to move this object during this event.  Doing so may
+           cause colliders that should be stuck to this object to become
+           un-stuck, or cause this object to pass through colliders it
+           should be pushing.
+        """
+        pass
+
+    def event_physics_collision_right(self, other, move_loss):
+        """
+        Called when the right side of the wall collides with a collider,
+        wall, or slope in the sense of the physics system, rather than
+        in the sense of SGE collision detection.  See the documentation
+        for :meth:`xsge_physics.Collider.event_physics_collision_left`
+        for more information.
+
+        .. note::
+
+           Due to the way movement is implemented in this class, it is
+           not safe to move this object during this event.  Doing so may
+           cause colliders that should be stuck to this object to become
+           un-stuck, or cause this object to pass through colliders it
+           should be pushing.
+        """
+        pass
+
+    def event_physics_collision_top(self, other, move_loss):
+        """
+        Called when the top side of the wall collides with a collider,
+        wall, or slope in the sense of the physics system, rather than
+        in the sense of SGE collision detection.  See the documentation
+        for :meth:`xsge_physics.Collider.event_physics_collision_left`
+        for more information.
+
+        .. note::
+
+           Due to the way movement is implemented in this class, it is
+           not safe to move this object during this event.  Doing so may
+           cause colliders that should be stuck to this object to become
+           un-stuck, or cause this object to pass through colliders it
+           should be pushing.
+        """
+        pass
+
+    def event_physics_collision_bottom(self, other, move_loss):
+        """
+        Called when the bottom side of the wall collides with a
+        collider, wall, or slope in the sense of the physics system,
+        rather than in the sense of SGE collision detection.  See the
+        documentation for
+        :meth:`xsge_physics.Collider.event_physics_collision_left` for
+        more information.
+
+        .. note::
+
+           Due to the way movement is implemented in this class, it is
+           not safe to move this object during this event.  Doing so may
+           cause colliders that should be stuck to this object to become
+           un-stuck, or cause this object to pass through colliders it
+           should be pushing.
+        """
+        pass
